using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace BreadAndCheese {
    /// <summary>
    /// To jest gwny typ Twojej gry.
    /// </summary>
    public class BreadAndCheeseGame : Microsoft.Xna.Framework.Game {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public struct BatSpriteStruct {
            private Texture2D spriteTexture;
            private Rectangle spriteRectangle;
            private float x;
            private float y;
            private float xSpeed;
            private float ySpeed;

            private float minDisplayX;
            private float maxDisplayX;
            private float minDisplayY;
            private float maxDisplayY;

            /// <summary>
            /// aduje tekstur do sprajtu paki.
            /// </summary>
            /// <param name="inSpriteTexture">tekstura do zaadowania</param>
            public void LoadTexture(Texture2D inSpriteTexture) {
                spriteTexture = inSpriteTexture;
            }

            /// <summary>
            /// Metoda rozpoczyna gr. Wyznacza rozmiary wszystkich obiektw na 
            /// ekranie i umieszcza na pocztkowych pozycjach.
            /// </summary>
            /// <param name="widthFactor">Rozmiar obiektu w grze wyraony jako uamek szerokoci
            /// ekranu.</param>
            /// <param name="ticksToCrossScreen">Szybko poruszania si obiektu wyraona w liczbie taktw (1/60 sekundy) 
            /// potrzebnych do pokonania caego ekranu.</param>
            /// <param name="inMinDisplayX">minimalna warto X</param>
            /// <param name="inMaxDisplayX">maksymalna warto X</param>
            /// <param name="inMinDisplayY">minimalna warto Y</param>
            /// <param name="inMaxDisplayY">maksymalna warto Y</param>
            /// <param name="initialX">pocztkowa pozycja paki (X)</param>
            /// <param name="initialY">pocztkowa pozycja paki (Y)</param>

            public void StartGame(
                    float widthFactor,
                    float ticksToCrossScreen,
                    float inMinDisplayX,
                    float inMaxDisplayX,
                    float inMinDisplayY,
                    float inMaxDisplayY,
                    float initialX,
                    float initialY) {
                minDisplayX = inMinDisplayX;
                minDisplayY = inMinDisplayY;
                maxDisplayX = inMaxDisplayX;
                maxDisplayY = inMaxDisplayY;

                float displayWidth = maxDisplayX - minDisplayX;

                spriteRectangle.Width = (int) ((displayWidth * widthFactor) + 0.5f);
                float aspectRatio =
                        (float) spriteTexture.Width / spriteTexture.Height;
                spriteRectangle.Height =
                        (int) ((spriteRectangle.Width / aspectRatio) + 0.5f);
                x = initialX;
                y = initialY;
                xSpeed = displayWidth / ticksToCrossScreen;
                ySpeed = xSpeed;
            }

            /// <summary>
            /// Rysuje pak na ekranie.
            /// </summary>
            /// <param name="spriteBatch">Obiekt klasy SpriteBatch uywany do rysowania na ekranie</param>
            public void Draw(SpriteBatch spriteBatch) {
                spriteBatch.Draw(spriteTexture, spriteRectangle, Color.White);
            }

            /// <summary>
            /// Aktualizuje pooenie paki zalenie od stanu pada.
            /// </summary>
            public void Update(BreadAndCheeseGame game) {
                GamePadState gamePad1 = GamePad.GetState(PlayerIndex.One);

                x = x + (xSpeed * gamePad1.ThumbSticks.Left.X);
                y = y - (ySpeed * gamePad1.ThumbSticks.Left.Y);
                spriteRectangle.X = (int) x;
                spriteRectangle.Y = (int) y;
            }

            /// <summary>
            /// Sprawdza ewentualne kolizje pomidzy pak a pozostaymi obiektami.
            /// </summary>
            /// <param name="target">Prostokt reprezentujcy pooenie innego obiektu.</param>
            /// <returns>true, jeli paka koliduje z danym obiektem</returns>
            public bool CheckCollision(Rectangle target) {
                return spriteRectangle.Intersects(target);
            }
        }

        public struct BallSpriteStruct {
            private Texture2D spriteTexture;
            private Rectangle spriteRectangle;
            private float x;
            private float y;
            private float xSpeed;
            private float ySpeed;

            private float minDisplayX;
            private float maxDisplayX;
            private float minDisplayY;
            private float maxDisplayY;

            /// <summary>
            /// aduje tekstur do sprajtu piki.
            /// </summary>
            /// <param name="inSpriteTexture">tekstura do zaadowania</param>
            public void LoadTexture(Texture2D inSpriteTexture) {
                spriteTexture = inSpriteTexture;
            }

            /// <summary>
            /// Metoda rozpoczyna gr. Wyznacza rozmiary wszystkich obiektw na 
            /// ekranie i umieszcza na pocztkowych pozycjach.
            /// </summary>
            /// <param name="widthFactor">Rozmiar obiektu w grze wyraony jako uamek szerokoci
            /// ekranu.</param>
            /// <param name="ticksToCrossScreen">Szybko poruszania si obiektu wyraona w liczbie taktw (1/60 sekundy) 
            /// potrzebnych do pokonania caego ekranu.</param>
            /// <param name="inMinDisplayX">minimalna warto X</param>
            /// <param name="inMaxDisplayX">maksymalna warto X</param>
            /// <param name="inMinDisplayY">minimalna warto Y</param>
            /// <param name="inMaxDisplayY">maksymalna warto Y</param>
            /// <param name="initialX">pocztkowa pozycja piki (X)</param>
            /// <param name="initialY">pocztkowa pozycja piki (Y)</param>

            public void StartGame(
                    float widthFactor,
                    float ticksToCrossScreen,
                    float inMinDisplayX,
                    float inMaxDisplayX,
                    float inMinDisplayY,
                    float inMaxDisplayY,
                    float initialX,
                    float initialY) {
                minDisplayX = inMinDisplayX;
                minDisplayY = inMinDisplayY;
                maxDisplayX = inMaxDisplayX;
                maxDisplayY = inMaxDisplayY;

                float displayWidth = maxDisplayX - minDisplayX;

                spriteRectangle.Width = (int) ((displayWidth * widthFactor) + 0.5f);
                float aspectRatio =
                        (float) spriteTexture.Width / spriteTexture.Height;
                spriteRectangle.Height =
                        (int) ((spriteRectangle.Width / aspectRatio) + 0.5f);
                x = initialX;
                y = initialY;
                xSpeed = displayWidth / ticksToCrossScreen;
                ySpeed = xSpeed;
            }

            /// <summary>
            /// Rysuje pik na ekranie.
            /// </summary>
            /// <param name="spriteBatch">Obiekt klasy SpriteBatch uywany do rysowania na ekranie</param>
            public void Draw(SpriteBatch spriteBatch) {
                spriteBatch.Draw(spriteTexture, spriteRectangle, Color.White);
            }

            /// <summary>
            /// Aktualizuje pooenie piki. Obsuguje kolizje z pak.
            /// </summary>
            /// <param name="game">Gra, w ktrej wystpuje ta pika</param>
            public void Update(BreadAndCheeseGame game) {
                x = x + xSpeed;
                y = y + ySpeed;

                // Ustawia nowe pooenie prostokta sprajtu:
                spriteRectangle.X = (int) (x + 0.5f);
                spriteRectangle.Y = (int) (y + 0.5f);

                // Sprawdza, czy pika trafia w pak
                if (game.BreadBat.CheckCollision(spriteRectangle)) {
                    // Paka trafia w pik.
                    ySpeed = ySpeed * -1;
                }

                if (x + spriteRectangle.Width >= maxDisplayX) {
                    // Pika zostaa trafiona z prawej strony.
                    xSpeed = Math.Abs(xSpeed) * -1;
                }

                if (x <= minDisplayX) {
                    // Pika zostaa trafiona z lewej strony.
                    xSpeed = Math.Abs(xSpeed);
                }

                // Sprawdza, czy pika trafia w doln krawd ekranu.
                if (y + spriteRectangle.Height >= maxDisplayY) {
                    // Pika zderzya si z doln krawdzi. Powoduje utrat ycia.
                    ySpeed = Math.Abs(ySpeed) * -1;
                    game.LoseLife();
                }

                if (y <= minDisplayY) {
                    // Pika uderzya w grn cz ekranu.
                    ySpeed = Math.Abs(ySpeed);
                }
            }
        }

        // wiat gry
        public BatSpriteStruct BreadBat;
        public BallSpriteStruct CheeseBall;

#region Wartoci reprezentujce wymiary ekranu

        float displayWidth;
        float displayHeight;
        float overScanPercentage = 10.0f;
        float minDisplayX;
        float maxDisplayX;
        float minDisplayY;
        float maxDisplayY;

        private void setScreenSizes() {
            displayWidth = graphics.GraphicsDevice.Viewport.Width;
            displayHeight = graphics.GraphicsDevice.Viewport.Height;
            float xOverscanMargin = getPercentage(overScanPercentage, displayWidth) / 2.0f;
            float yOverscanMargin = getPercentage(overScanPercentage, displayHeight) / 2.0f;

            minDisplayX = xOverscanMargin;
            minDisplayY = yOverscanMargin;

            maxDisplayX = displayWidth - xOverscanMargin;
            maxDisplayY = displayHeight - yOverscanMargin;
        }


#endregion

#region Metody pomocnicze

        /// <summary>
        /// Oblicza wartoci procentowe
        /// </summary>
        /// <param name="percentage">procent, ktry ma zosta wyznaczony</param>
        /// <param name="inputValue">warto do przeliczenia</param>
        /// <returns>warto wynikowa</returns>

        float getPercentage(float percentage, float inputValue) {
            return (inputValue * percentage) / 100;
        }

#endregion

        /// <summary>
        /// Metoda jest wywoywana z chwil utraty ycia. Gra koczy si w sytuacji, gdy gracz 
        /// wykorzysta wszystkie dostpne ycia.
        /// </summary>
        public void LoseLife() {
        }

        /// <summary>
        /// Metoda wywoywana w odpowiedzi na zdarzenie powodujce zmian punktacji.
        /// </summary>
        /// <param name="update">Zmiana liczby punktw</param>
        public void UpdateScore(int update) {
        }


        public BreadAndCheeseGame() {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Umoliwia ewentualn inicjalizacj przed uruchomieniem waciwej gry.
        /// W tym miejscu mona odnale wszystkie wymagane zasoby i zaadowa treci
        /// related content. Wywoanie metody base.Initialize spowoduje przeszukanie wszystkich komponentw
        /// i ich inicjalizacj.
        /// </summary>
        protected override void Initialize() {
            setScreenSizes();

            base.Initialize();
        }

        /// <summary>
        /// Metoda LoadContent jest wywoywana tylko raz dla caej gry i jako taka jest waciwym miejscem
        /// dla kodu adujcego tre.
        /// </summary>
        protected override void LoadContent() {
            // Tworzy nowy obiekt klasy SpriteBatch, ktrego mona uywa do rysowania tekstur.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            BreadBat.LoadTexture(Content.Load<Texture2D > ("Images/Bread"));
            CheeseBall.LoadTexture(Content.Load<Texture2D > ("Images/Cheese"));

            CheeseBall.StartGame(
                    0.07f, // szeroko sera wynosi 0,07 szerokoci ekranu
                    200, // ser potrzebuje 200 taktw na pokonanie caego ekranu
                    minDisplayX, maxDisplayX, minDisplayY, maxDisplayY,
                    displayWidth / 4, // jedna czwarta szerokoci ekranu
                    displayHeight / 4); // jedna czwarta wysokoci ekranu od grnej krawdzi

            BreadBat.StartGame(
                    0.166f, // szeroko chleba wynosi 0,166 szerokoci ekranu
                    150, // pokonanie ekranu zajmuje 150 taktw
                    minDisplayX, maxDisplayX, minDisplayY, maxDisplayY,
                    displayWidth / 2, // na pocztku znajduje si na rodku ekranu (w poziomie)
                    displayHeight / 2); // na pocztku znajduje si na rodku ekranu (w pionie)
        }

        /// <summary>
        /// Metoda UnloadContent jest wywoywana tylko raz dla caej gry i jako taka jest waciwym miejscem
        /// dla kodu usuwajcego ca tre z pamici.
        /// </summary>
        protected override void UnloadContent() {
            // TODO: Naley usun ca tre, ktr nie zarzdza ContentManager.
        }

        /// <summary>
        /// Umoliwia grze wykonywanie logiki zwizanej z aktualizacj wiata gry,
        /// sprawdzaniem kolizji, pobieraniem danych wejciowych czy odtwarzaniem dwikw.
        /// </summary>
        /// <param name="gameTime">Udostpnia wartoci reprezentujce biecy czas.</param>
        protected override void Update(GameTime gameTime) {
            GamePadState gamePad1 = GamePad.GetState(PlayerIndex.One);

            if (gamePad1.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            BreadBat.Update(this);
            CheeseBall.Update(this);

            base.Update(gameTime);
        }

        /// <summary>
        /// Ta metoda jest wywoywana w momencie, w ktrym gra musi narysowa swj wiat.
        /// </summary>
        /// <param name="gameTime">Udostpnia wartoci reprezentujce biecy czas.</param>
        protected override void Draw(GameTime gameTime) {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            BreadBat.Draw(spriteBatch);

            CheeseBall.Draw(spriteBatch);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
